/* Generated by Cython 0.29.21 */

#ifndef __PYX_HAVE__thinqpbo___qpbo
#define __PYX_HAVE__thinqpbo___qpbo

#include "Python.h"
struct PyObject_QPBOInt;
struct PyObject_QPBOFloat;
struct PyObject_QPBODouble;

/* "thinqpbo/src/_qpbo.pyx":6
 * 
 * 
 * cdef public class QPBOInt[object PyObject_QPBOInt, type QPBOInt]:             # <<<<<<<<<<<<<<
 *     cdef QPBO[int]* c_qpbo
 * 
 */
struct PyObject_QPBOInt {
  PyObject_HEAD
  QPBO<int>  *c_qpbo;
};

/* "thinqpbo/src/_qpbo.pyx":195
 * 
 * 
 * cdef public class QPBOFloat[object PyObject_QPBOFloat, type QPBOFloat]:             # <<<<<<<<<<<<<<
 *     cdef QPBO[float]* c_qpbo
 * 
 */
struct PyObject_QPBOFloat {
  PyObject_HEAD
  QPBO<float>  *c_qpbo;
};

/* "thinqpbo/src/_qpbo.pyx":384
 * 
 * 
 * cdef public class QPBODouble[object PyObject_QPBODouble, type QPBODouble]:             # <<<<<<<<<<<<<<
 *     cdef QPBO[double]* c_qpbo
 * 
 */
struct PyObject_QPBODouble {
  PyObject_HEAD
  QPBO<double>  *c_qpbo;
};

#ifndef __PYX_HAVE_API__thinqpbo___qpbo

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#ifndef DL_IMPORT
  #define DL_IMPORT(_T) _T
#endif

__PYX_EXTERN_C DL_IMPORT(PyTypeObject) QPBOInt;
__PYX_EXTERN_C DL_IMPORT(PyTypeObject) QPBOFloat;
__PYX_EXTERN_C DL_IMPORT(PyTypeObject) QPBODouble;

#endif /* !__PYX_HAVE_API__thinqpbo___qpbo */

/* WARNING: the interface of the module init function changed in CPython 3.5. */
/* It now returns a PyModuleDef instance instead of a PyModule instance. */

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_qpbo(void);
#else
PyMODINIT_FUNC PyInit__qpbo(void);
#endif

#endif /* !__PYX_HAVE__thinqpbo___qpbo */
